use std::collections::HashSet;
use std::path::Path;
use std::{fs, io};

use futures::future::Future;
use futures::stream::Stream;
use futures::{Async, Poll};
use hyper::client::{HttpConnector, ResponseFuture};
use hyper::{Body, Client, Response};
use hyper_tls::HttpsConnector;
use serde::de;

use crate::twitter::Twitter;

type HttpsClient = Client<HttpsConnector<HttpConnector>, hyper::Body>;

#[derive(Serialize, Deserialize)]
pub struct PackageListResult {
    pub help: String,
    pub success: bool,
    pub result: Vec<String>,
}

impl PackageListResult {
    fn deserialize(req: Response<Body>) -> serde_json::Result<Self>
    where
        for<'de> Self: de::Deserialize<'de>,
    {
        let (_, body) = req.into_parts();
        // TODO handle error
        let body = serde_json::from_slice(&body.concat2().wait().unwrap())?;
        Ok(body)
    }
}

#[derive(Debug)]
pub struct CkanAPI {
    /// hyper http client to build requests with.
    http: HttpsClient,
    twitter: Twitter,
}

impl CkanAPI {
    pub fn new(twitter: Twitter) -> Self {
        let http = {
            let connector = HttpsConnector::new(4).unwrap();

            Client::builder().build(connector)
        };

        CkanAPI { http, twitter }
    }

    pub fn getPackageList(&self) -> impl Future<Item = (), Error = io::Error> + Send {
        GetPackageListFuture::new(self.http.clone(), self.twitter.clone())
    }
}

pub struct GetPackageListFuture {
    response: ResponseFuture,
    twitter: Twitter,
}

impl GetPackageListFuture {
    fn new(http: HttpsClient, twitter: Twitter) -> Self {
        let uri = "https://opendata.schleswig-holstein.de/api/3/action/package_list"
            .parse()
            .unwrap();

        Self {
            response: http.get(uri),
            twitter,
        }
    }
}

impl Future for GetPackageListFuture {
    type Item = ();
    type Error = io::Error;

    fn poll(&mut self) -> Poll<Self::Item, Self::Error> {
        match self.response.poll() {
            Ok(Async::Ready(res)) => {
                let data: PackageListResult = PackageListResult::deserialize(res).unwrap();
                let mut added_datasets: HashSet<String> = HashSet::new();
                let mut removed_datasets: HashSet<String> = HashSet::new();
                if !Path::new("./data/").exists() {
                    fs::create_dir_all("./data/");
                }
                if Path::new("./data/latestPackageList.json").exists() {
                    let cache_file: String =
                        fs::read_to_string("./data/latestPackageList.json").unwrap();
                    let cache: HashSet<String> =
                        serde_json::from_str::<Vec<String>>(cache_file.as_str())
                            .unwrap()
                            .iter()
                            .cloned()
                            .collect();
                    let newdata: HashSet<String> = data.result.iter().cloned().collect();

                    removed_datasets = cache.difference(&newdata).cloned().collect();
                    added_datasets = newdata.difference(&cache).cloned().collect();
                }
                let serialized = serde_json::to_string(&data.result).unwrap();
                fs::write("./data/latestPackageList.json", serialized)
                    .expect("Unable to write latestPackageList");

                self.twitter
                    .post_changed_datasets(added_datasets, removed_datasets);
                Ok(Async::Ready(()))
            }
            Ok(Async::NotReady) => Ok(Async::NotReady),
            Err(e) => {
                println!("failed to get response: {}", e);
                Ok(Async::Ready(()))
            }
        }
    }
}
